@@ -30,22 +30,28 @@
 #include "translate.h"
 
 
+const unsigned char pause_phonemes[8] = {0, phonPAUSE_VSHORT, phonPAUSE_SHORT, phonPAUSE, phonPAUSE_LONG, phonGLOTTALSTOP, phonPAUSE_LONG, phonPAUSE_LONG};
 
 
-int Translator::ChangePhonemes(PHONEME_LIST2 *phlist, int n_ph, int index, PHONEME_TAB *ph, CHANGEPH *ch)
-{//======================================================================================================
+extern int n_ph_list2;
+extern PHONEME_LIST2 ph_list2[N_PHONEME_LIST];	// first stage of text->phonemes
+
+
+
+static int ChangePhonemes(Translator *tr, PHONEME_LIST2 *phlist, int n_ph, int index, PHONEME_TAB *ph, CHANGEPH *ch)
+{//=================================================================================================================
 // Called for each phoneme in the phoneme list, to allow a language to make changes
-// flags: bit 0=1 last phoneme in a word
-//        bit 1=1 this is the highest stressed vowel in the current word
-//        bit 2=1 after the highest stressed vowel in the current word
-//        bit 3=1 the phonemes were specified explicitly, or found from an entry in the xx_list dictionary
 // ph     The current phoneme
+
+	if(tr->translator_name == L('r','u'))
+		return(ChangePhonemes_ru(tr, phlist, n_ph, index, ph, ch));
+
 	return(0);
 }
 
 
-int Translator::SubstitutePhonemes(PHONEME_LIST2 *plist_out)
-{//=========================================================
+static int SubstitutePhonemes(Translator *tr, PHONEME_LIST2 *plist_out)
+{//====================================================================
 // Copy the phonemes list and perform any substitutions that are required for the
 // current voice
 	int ix;
@@ -80,7 +86,7 @@ int Translator::SubstitutePhonemes(PHONEME_LIST2 *plist_out)
 			if((plist2+1)->sourceix || ((next != 0) && (next->type == phPAUSE)))
 				word_end = 1;        // this phoneme is the end of a word
 	
-			if(langopts.phoneme_change != 0)
+			if(tr->langopts.phoneme_change != 0)
 			{
 				// this language does changes to phonemes after translation
 
@@ -137,7 +143,7 @@ int Translator::SubstitutePhonemes(PHONEME_LIST2 *plist_out)
 				ch.vowel_this = syllable;
 				ch.vowel_stressed = syllable_stressed;
 
-				ChangePhonemes(ph_list2, n_ph_list2, ix, phoneme_tab[ph_list2[ix].phcode], &ch);
+				ChangePhonemes(tr, ph_list2, n_ph_list2, ix, phoneme_tab[ph_list2[ix].phcode], &ch);
 			}
 	
 			// check whether a Voice has specified that we should replace this phoneme
@@ -155,6 +161,8 @@ int Translator::SubstitutePhonemes(PHONEME_LIST2 *plist_out)
 	
 					// substitute the replacement phoneme
 					plist2->phcode = replace_phonemes[k].new_ph;
+					if((plist2->stress > 1) && (phoneme_tab[plist2->phcode]->phflags & phUNSTRESSED))
+						plist2->stress = 0;   // the replacement must be unstressed
 					break;
 				}
 			}
@@ -173,40 +181,42 @@ int Translator::SubstitutePhonemes(PHONEME_LIST2 *plist_out)
 
 
 
-void Translator::MakePhonemeList(int post_pause, int start_sentence)
-{//============================================================================================
+void MakePhonemeList(Translator *tr, int post_pause, int start_sentence)
+{//=====================================================================
 
 	int  ix=0;
 	int  j;
 	int  insert_ph = 0;
-	int  insert_synthflags = 0;
 	PHONEME_LIST *phlist;
 	PHONEME_TAB *ph;
 	PHONEME_TAB *prev, *next, *next2;
 	int unstress_count = 0;
-	int word_has_stress = 0;
+	int word_stress = 0;
 	int switched_language = 0;
 	int max_stress;
 	int voicing;
 	int regression;
 	int end_sourceix;
 	int alternative;
+	int first_vowel=0;   // first vowel in a word
 	PHONEME_LIST2 ph_list3[N_PHONEME_LIST];
 
 	static PHONEME_LIST2 ph_list2_null = {0,0,0,0,0};
 	PHONEME_LIST2 *plist2 = &ph_list2_null;
+	PHONEME_LIST2 *plist2_inserted = NULL;
 
+	plist2 = ph_list2;
 	phlist = phoneme_list;
-	end_sourceix = ph_list2[n_ph_list2-1].sourceix;
+	end_sourceix = plist2[n_ph_list2-1].sourceix;
 
 	// is the last word of the clause unstressed ?
 	max_stress = 0;
-	for(j=n_ph_list2-3; j>=0; j--)
+	for(j = n_ph_list2-3; j>=0; j--)
 	{
 		// start with the last phoneme (before the terminating pauses) and move forwards
-		if((ph_list2[j].stress & 0x7f) > max_stress)
-			max_stress = ph_list2[j].stress & 0x7f;
-		if(ph_list2[j].sourceix != 0)
+		if((plist2[j].stress & 0x7f) > max_stress)
+			max_stress = plist2[j].stress & 0x7f;
+		if(plist2[j].sourceix != 0)
 			break;
 	}
 	if(max_stress < 4)
@@ -214,12 +224,12 @@ void Translator::MakePhonemeList(int post_pause, int start_sentence)
 		// the last word is unstressed, look for a previous word that can be stressed
 		while(--j >= 0)
 		{
-			if(ph_list2[j].stress & 0x80)  // dictionary flags indicated that this stress can be promoted
+			if(plist2[j].synthflags & SFLAG_PROMOTE_STRESS)  // dictionary flags indicated that this stress can be promoted
 			{
-				ph_list2[j].stress = 4;   // promote to stressed
+				plist2[j].stress = 4;   // promote to stressed
 				break;
 			}
-			if((ph_list2[j].stress & 0x7f) >= 4)
+			if(plist2[j].stress >= 4)
 			{
 				// found a stressed syllable, so stop looking
 				break;
@@ -227,7 +237,7 @@ void Translator::MakePhonemeList(int post_pause, int start_sentence)
 		}
 	}
 
-	if((regression = langopts.param[LOPT_REGRESSIVE_VOICING]) != 0)
+	if((regression = tr->langopts.param[LOPT_REGRESSIVE_VOICING]) != 0)
 	{
 		// set consonant clusters to all voiced or all unvoiced
 		// Regressive
@@ -236,7 +246,7 @@ void Translator::MakePhonemeList(int post_pause, int start_sentence)
 
 		for(j=n_ph_list2-1; j>=0; j--)
 		{
-			ph = phoneme_tab[ph_list2[j].phcode];
+			ph = phoneme_tab[plist2[j].phcode];
 			if(ph == NULL)
 				continue;
 
@@ -263,7 +273,7 @@ void Translator::MakePhonemeList(int post_pause, int start_sentence)
 				else
 				if((voicing==2) && ((ph->phflags & phALTERNATIVE)==phSWITCHVOICING))
 				{
-					ph_list2[j].phcode = ph->alternative_ph;  // change to voiced equivalent
+					plist2[j].phcode = ph->alternative_ph;  // change to voiced equivalent
 				}
 			}
 			else
@@ -276,7 +286,7 @@ void Translator::MakePhonemeList(int post_pause, int start_sentence)
 				else
 				if((voicing==1) && ((ph->phflags & phALTERNATIVE)==phSWITCHVOICING))
 				{
-					ph_list2[j].phcode = ph->alternative_ph;  // change to unvoiced equivalent
+					plist2[j].phcode = ph->alternative_ph;  // change to unvoiced equivalent
 				}
 			}
 			else
@@ -292,7 +302,7 @@ void Translator::MakePhonemeList(int post_pause, int start_sentence)
 					voicing = 0;
 				}
 			}
-			if((regression & 0x4) && (ph_list2[j].sourceix))
+			if((regression & 0x4) && (plist2[j].sourceix))
 			{
 				// stop propagation at a word boundary
 				voicing = 0;
@@ -300,13 +310,13 @@ void Translator::MakePhonemeList(int post_pause, int start_sentence)
 		}
 	}
 
-	n_ph_list2 = SubstitutePhonemes(ph_list3) - 2;
+	n_ph_list2 = SubstitutePhonemes(tr,ph_list3) - 2;
 
 	// transfer all the phonemes of the clause into phoneme_list
 	ph = phoneme_tab[phonPAUSE];
 	switched_language = 0;
 
-	for(j=0; insert_ph || ((j<n_ph_list2) && (ix < N_PHONEME_LIST-3)); j++)
+	for(j=0; insert_ph || ((j < n_ph_list2) && (ix < N_PHONEME_LIST-3)); j++)
 	{
 		prev = ph;
 
@@ -315,13 +325,16 @@ void Translator::MakePhonemeList(int post_pause, int start_sentence)
 		if(insert_ph != 0)
 		{
 			// we have a (linking) phoneme which we need to insert here
-			j--;
 			next = phoneme_tab[plist2->phcode];      // this phoneme, i.e. after the insert
-			plist2 = &ph_list2_null;
+
+			// re-use the previous entry for the inserted phoneme.
+			// That's OK because we don't look backwards from plist2
+			j--;
+			plist2 = plist2_inserted = &ph_list3[j];
+			memset(plist2, 0, sizeof(*plist2));
+			plist2->phcode = insert_ph;
 			ph = phoneme_tab[insert_ph];
-			plist2->synthflags = insert_synthflags;
 			insert_ph = 0;
-			insert_synthflags = 0;
 		}
 		else
 		{
@@ -338,7 +351,22 @@ void Translator::MakePhonemeList(int post_pause, int start_sentence)
 		}
 
 		if(plist2->sourceix)
-			word_has_stress = 0;   // start of a word
+		{
+			// start of a word
+			int k;
+			word_stress = 0;
+			first_vowel = 1;
+
+			// find the highest stress level in this word
+			for(k=j+1; k < n_ph_list2; k++)
+			{
+				if(ph_list3[k].sourceix)
+					break;   // start of the next word
+
+				if(ph_list3[k].stress > word_stress)
+					word_stress = ph_list3[k].stress;
+			}
+		}
 
 		if(ph == NULL) continue;
 
@@ -353,7 +381,7 @@ void Translator::MakePhonemeList(int post_pause, int start_sentence)
 				{
 					// in a sequence of unstressed syllables, reduce alternate syllables to 'diminished'
                // stress.  But not for the last phoneme of a stressed word
-					if((langopts.stress_flags & 0x2) || (word_has_stress && ((plist2+1)->sourceix!=0)))
+					if((tr->langopts.stress_flags & 0x2) || ((word_stress > 3) && ((plist2+1)->sourceix!=0)))
 					{
 						// An unstressed final vowel of a stressed word
 						unstress_count=1;    // try again for next syllable
@@ -367,15 +395,13 @@ void Translator::MakePhonemeList(int post_pause, int start_sentence)
 			else
 			{
 				unstress_count = 0;
-				if(plist2->stress > 3)
-					word_has_stress = 1;   // word has a primary or a secondary stress
 			}
 		}
 
+		alternative = 0;
+
 		if(ph->alternative_ph > 0)
 		{
-			alternative = 0;
-
 			switch(ph->phflags & phALTERNATIVE)
 			{
 			// This phoneme changes if vowel follows, or doesn't follow, depending on its phNOTFOLLOWS flag
@@ -384,6 +410,14 @@ void Translator::MakePhonemeList(int post_pause, int start_sentence)
 					alternative = ph->alternative_ph;
 				break;
 
+			case phBEFORENOTVOWEL2:    // LANG=tr
+				if(((plist2+1)->sourceix != 0) ||
+               ((next->type != phVOWEL) && ((phoneme_tab[(plist2+2)->phcode]->type != phVOWEL) || ((plist2+2)->sourceix != 0))))
+				{
+					alternative = ph->alternative_ph;
+				}
+				break;
+
 			case phBEFOREVOWELPAUSE:
 				if((next->type == phVOWEL) || (next->type == phPAUSE))
 					alternative = ph->alternative_ph;
@@ -393,29 +427,41 @@ void Translator::MakePhonemeList(int post_pause, int start_sentence)
 				if(next->type == phVOWEL)
 					alternative = ph->alternative_ph;
 				break;
-			}
 
-			if(alternative == 1)
-				continue;    // NULL phoneme, discard
-
-			if(alternative > 1)
-			{
-				PHONEME_TAB *ph2;
-				ph2 = ph;
-				ph = phoneme_tab[alternative];
-
-				if(ph->type == phVOWEL)
+			case phBEFORE_R:
+				if(next->phflags & phRHOTIC)
 				{
-					plist2->synthflags |= SFLAG_SYLLABLE;
-					if(ph2->type != phVOWEL)
-						plist2->stress = 0;   // change from non-vowel to vowel, make sure it's unstressed
+					alternative = ph->alternative_ph;
 				}
-				else
-					plist2->synthflags &= ~SFLAG_SYLLABLE;
+				break;
 			}
 		}
+		if(ph->phflags & phBEFOREPAUSE)
+		{
+			if(next->type == phPAUSE)
+				alternative = ph->link_out;   // replace with the link_out phoneme
+		}
 
-		if(langopts.param[LOPT_REDUCE_T])
+		if(alternative == 1)
+			continue;    // NULL phoneme, discard
+
+		if(alternative > 1)
+		{
+			PHONEME_TAB *ph2;
+			ph2 = ph;
+			ph = phoneme_tab[alternative];
+
+			if(ph->type == phVOWEL)
+			{
+				plist2->synthflags |= SFLAG_SYLLABLE;
+				if(ph2->type != phVOWEL)
+					plist2->stress = 0;   // change from non-vowel to vowel, make sure it's unstressed
+			}
+			else
+				plist2->synthflags &= ~SFLAG_SYLLABLE;
+		}
+
+		if(tr->langopts.param[LOPT_REDUCE_T])
 		{
 			if((ph->mnemonic == 't') && (plist2->sourceix == 0) && ((prev->type == phVOWEL) || (prev->mnemonic == 'n')))
 			{
@@ -426,22 +472,52 @@ void Translator::MakePhonemeList(int post_pause, int start_sentence)
 			}
 		}
 
-		if((ph->reduce_to != 0) && (ph->type != phVOWEL) && !(plist2->synthflags & SFLAG_DICTIONARY))
-		{
-			// reduction for vowels has already been done in SetWordStress
-			int reduce_level;
 
-			if(next->type == phVOWEL)
+		while((ph->reduce_to != 0) && (!(plist2->synthflags & SFLAG_DICTIONARY)  || (tr->langopts.param[LOPT_REDUCE] & 1)))
+		{
+			int reduce_level;
+			int stress_level;
+			int reduce = 0;
+
+			reduce_level = (ph->phflags >> 28) & 7;
+
+			if(ph->type == phVOWEL)
 			{
-				reduce_level = (ph->phflags >> 28) & 7;
-				if((&plist2[1])->stress < reduce_level)
-				{
-					// look at the stress of the following vowel
-					ph = phoneme_tab[ph->reduce_to];
-				}
+				stress_level = plist2->stress;
 			}
+			else
+			{
+				// consonant, get stress from the following vowel
+				if(next->type == phVOWEL)
+					stress_level = (plist2+1)->stress;
+				else
+					break;
+			}
+
+			if((stress_level == 1) && (first_vowel))
+				stress_level = 0;   // ignore 'dimished' stress on first syllable
+
+			if(stress_level == 1)
+				reduce = 1;    // stress = 'reduced'
+
+			if(stress_level < reduce_level)
+				reduce =1;
+
+			if((word_stress < 4) && (tr->langopts.param[LOPT_REDUCE] & 0x2) && (stress_level >= word_stress))
+			{
+				// don't reduce the most stressed syllable in an unstressed word
+				reduce = 0;
+			}
+
+			if(reduce)
+				ph = phoneme_tab[ph->reduce_to];
+			else
+				break;
 		}
 
+		if(ph->type == phVOWEL)
+			first_vowel = 0;
+
 		if((plist2+1)->synthflags & SFLAG_LENGTHEN)
 		{
 			static char types_double[] = {phFRICATIVE,phVFRICATIVE,phNASAL,phLIQUID,0};
@@ -457,55 +533,63 @@ void Translator::MakePhonemeList(int post_pause, int start_sentence)
 		{
 			int x;
 
-			if(langopts.word_gap & 1)
+			if(tr->langopts.vowel_pause && (ph->type != phPAUSE))
 			{
-				insert_ph = phonPAUSE_VSHORT;
-			}
-			if(langopts.vowel_pause && (ph->type != phPAUSE) && (next->type == phVOWEL))
-			{
-				if(langopts.vowel_pause & 0x04)
+
+				if((ph->type != phVOWEL) && (tr->langopts.vowel_pause & 0x200))
 				{
-					// break before a word which starts with a vowel
-					insert_ph = phonPAUSE_VSHORT;
+					// add a pause after a word which ends in a consonant
+					insert_ph = phonPAUSE_NOLINK;
 				}
 
-				if((ph->type == phVOWEL) && ((x = langopts.vowel_pause & 0x03) != 0))
+				if(next->type == phVOWEL)
 				{
-					// adjacent vowels over a word boundary
-					if(x == 2)
+					if((x = tr->langopts.vowel_pause & 0x0c) != 0)
+					{
+						// break before a word which starts with a vowel
+						if(x == 0xc)
+							insert_ph = phonPAUSE_NOLINK;
+						else
+							insert_ph = phonPAUSE_VSHORT;
+					}
+	
+					if((ph->type == phVOWEL) && ((x = tr->langopts.vowel_pause & 0x03) != 0))
+					{
+						// adjacent vowels over a word boundary
+						if(x == 2)
+							insert_ph = phonPAUSE_SHORT;
+						else
+							insert_ph = phonPAUSE_VSHORT;
+					}
+	
+					if(((plist2+1)->stress >= 4) && (tr->langopts.vowel_pause & 0x100))
+					{
+						// pause before a words which starts with a stressed vowel
 						insert_ph = phonPAUSE_SHORT;
-					else
-						insert_ph = phonPAUSE_VSHORT;
+					}
 				}
+			}
 
-				if(((plist2+1)->stress >= 4) && (langopts.vowel_pause & 0x08))
+			if(plist2 != plist2_inserted)
+			{
+				if((x = (tr->langopts.word_gap & 0x7)) != 0)
 				{
-					// pause before a words which starts with a stressed vowel
-					insert_ph = phonPAUSE_SHORT;
+					if((x > 1) || ((insert_ph != phonPAUSE_SHORT) && (insert_ph != phonPAUSE_NOLINK)))
+					{
+						// don't reduce the pause
+						insert_ph = pause_phonemes[x];
+					}
+				}
+				if(option_wordgap > 0)
+				{
+					insert_ph = phonPAUSE_LONG;
 				}
 			}
 		}
 
 		next2 = phoneme_tab[(plist2+2)->phcode];
 
-#ifdef deleted
-		if((ph->type != phVOWEL)  && (ph->type != phPAUSE)   //&& ((ph->phflags & phVOICED)==0)
-			&& (next->type == phLIQUID) && (next->mnemonic != ';') && (next2->type != phVOWEL))
-		{
-			// semi-vowel surrounded by consonants.  precede by a short schwa
-			insert_ph = phonSCHWA_SHORT;
-			insert_synthflags = SFLAG_SYLLABLE;
-		}
-
-		if((ph->type == phLIQUID) && (prev->type != phVOWEL) && (next->type != phVOWEL))
-		{
-			// semi-vowel surrounded by consonants.  precede by a short schwa
-			insert_ph = phonSCHWA_SHORT;
-			insert_synthflags = SFLAG_SYLLABLE;
-		}
-#endif
-
-		if((insert_ph == 0) && (ph->link_out != 0) && (((plist2+1)->synthflags & SFLAG_EMBEDDED)==0))
+		if((insert_ph == 0) && (ph->link_out != 0) && !(ph->phflags & phBEFOREPAUSE) && (((plist2+1)->synthflags & SFLAG_EMBEDDED)==0))
 		{
 			if(ph->phflags & phAPPENDPH)
 			{
@@ -517,7 +601,7 @@ void Translator::MakePhonemeList(int post_pause, int start_sentence)
 				}
 			}
 			else
-			if(((langopts.word_gap & 2)==0) || ((plist2+1)->sourceix == 0))
+			if(((tr->langopts.word_gap & 8)==0) || ((plist2+1)->sourceix == 0))
 			{
 				// This phoneme can be linked to a following vowel by inserting a linking phoneme
 				if(next->type == phVOWEL)
@@ -544,7 +628,7 @@ void Translator::MakePhonemeList(int post_pause, int start_sentence)
 		phlist[ix].type = ph->type;
 		phlist[ix].env = PITCHfall;          // default, can be changed in the "intonation" module
 		phlist[ix].synthflags = plist2->synthflags | switched_language;
-		phlist[ix].tone = plist2->stress & 0xf;
+		phlist[ix].stresslevel = plist2->stress & 0xf;
 		phlist[ix].tone_ph = plist2->tone_number;
 		phlist[ix].sourceix = 0;
 
@@ -565,6 +649,11 @@ void Translator::MakePhonemeList(int post_pause, int start_sentence)
 		}
 
 		phlist[ix].length = ph->std_length;
+		if((ph->code == phonPAUSE_LONG) && (option_wordgap > 0))
+		{
+			phlist[ix].ph = phoneme_tab[phonPAUSE_SHORT];
+			phlist[ix].length = option_wordgap*14;   // 10mS per unit at the default speed
+		}
 
 		if(ph->type==phVOWEL || ph->type==phLIQUID || ph->type==phNASAL || ph->type==phVSTOP || ph->type==phVFRICATIVE)
 		{
@@ -590,7 +679,7 @@ void Translator::MakePhonemeList(int post_pause, int start_sentence)
 	phlist[ix].length = 0;
 	phlist[ix].sourceix=0;
 	phlist[ix].synthflags = 0;
-   phlist[ix++].ph = phoneme_tab[phonPAUSE_VSHORT];
+   phlist[ix++].ph = phoneme_tab[phonPAUSE_SHORT];
 
 	n_phoneme_list = ix;
 }  // end of MakePhonemeList
